! Lorenz's solution using Runge-Kutta 4th order scheme
module params
double precision,parameter :: p = 10.0,r=32.0,b=2.666666667,del_t=0.01
integer, parameter :: nstop  = 500
double precision,dimension(3) :: xinit =(/1.0,3.0,5.0/)
double precision, dimension(nstop) :: tarray
double precision,dimension(3,0:nstop) :: x_save
end module params

! program to solve lorenz equation
subroutine lorenz_deriv(t,x,dxdt)
use params
double precision,dimension(3),intent(in) :: x
double precision,dimension(3),intent(out) :: dxdt
double precision :: t
dxdt(1) = -p*x(1) + p*x(2)
dxdt(2) = x(1)*(r-x(3)) - x(2)
dxdt(3) = x(1)*x(2) - b*x(3)
end subroutine lorenz_deriv
!
subroutine rk4(t,x,dt)
use params
double precision,intent(in) :: t,dt
double precision,intent(inout),dimension(3) :: x
double precision,dimension(3) :: f1,f2,f3,f4
call lorenz_deriv(t,x,f1)
f1 = dt*f1
!
call lorenz_deriv(t+0.5*dt,x+0.5*f1,f2)
f2 = dt*f2
!
call lorenz_deriv(t+0.5*dt,x+0.5*f2,f3)
f3 = dt*f3
!
call lorenz_deriv(t+dt,x+f3,f4)  
f4 = dt*f4
x = x +(f1 + 2*f2 + 2*f3 + f4)/6.0
end subroutine rk4


subroutine forward_model(x, x_allsteps)
use params
double precision, dimension(3) :: x
double precision, dimension(3,nstop) :: x_allsteps
double precision :: t
integer :: i
do i = 1,nstop
  t = i*del_t
  tarray(i) = t
  call rk4(t,x,del_t)
  x_allsteps(:,i) = x(:)
enddo
end subroutine forward_model



program lorenz
! Example in Huang and Yang's paper

use IFPORT
use params
double precision,dimension(3) :: x,dxdt
double precision,dimension(3,nstop) :: x_allsteps
include 'netcdf.inc'
integer :: ncid,timeid,timedimid,x1_id,x2_id,x3_id



ier = nf_create('lorenz_rk4_dp.nc',nf_clobber,ncid)
ier=NF_DEF_DIM(ncid,'time',nf_unlimited,timedimid)
ier=NF_DEF_VAR(ncid,'time',NF_DOUBLE,1,(/timedimid/),timeid)
! define dependent variables
!
ier=NF_DEF_VAR(ncid,'x1',NF_DOUBLE,1,(/timedimid/),x1_id)
ier=NF_DEF_VAR(ncid,'x2',NF_DOUBLE,1,(/timedimid/),x2_id)
ier=NF_DEF_VAR(ncid,'x3',NF_DOUBLE,1,(/timedimid/),x3_id)
!

ier=NF_ENDDEF(ncid)

x =  xinit 
call  forward_model(x,x_allsteps)
x_save(:,0) = xinit
x_save(:,1:nstop) = x_allsteps(:,1:nstop) 
!

do i = 1,nstop
! Do netcdf writes
  ier=NF_PUT_VARA_DOUBLE(ncid,x1_id,(/i/),(/1/),x_save(1,i))
  ier=NF_PUT_VARA_DOUBLE(ncid,x2_id,(/i/),(/1/),x_save(2,i))
  ier=NF_PUT_VARA_DOUBLE(ncid,x3_id,(/i/),(/1/),x_save(3,i))
  ier=NF_PUT_VARA_DOUBLE(ncid,timeid,(/i/),(/1/),tarray(i))
enddo

ier = NF_CLOSE(ncid)
stop
end

  

