! Subroutine to do power iteration with aitken acceleration
! to find largest real eigen value of A
! Will work only for real eigen values
! Let phi = x2
! See 226-234 ; Kincaid and Cheney, 1991
subroutine power_aitken(a,n,x,y,r,m,s)
integer,intent(in) :: n,m 
real,intent(in) :: a(n,n)
real,intent(inout) :: x(n),y(n),r(n),s(n)
real :: ymag
integer :: k
real :: eps = 1.0e-5,denom
do k = 1,m
  call matvec(a,x,y,n,n)
  r(k) = y(2)/x(2)
  call vec_mag(y,n,ymag)
  x = y/ymag
  write(*,*)'r,x',r(k),x
enddo
! The acceleration part
do k = 1,m-2
  denom = r(k+2) - 2.0*r(k+1) + r(k)
  if ( denom .ge. eps) then
    s(k) = (r(k)*r(k+2) - r(k+1)**2)/denom
    write(*,*)'r,s',r(k),s(k)
  endif
enddo
return
end
subroutine matvec(a,x,y,m,n)
real,intent(in) :: a(m,n),x(n)
integer,intent(in) :: m,n
real,intent(out) :: y(n)
integer :: i,k

do i = 1,m
  y(i) = 0.0
  do k = 1,n
    y(i) = y(i) + a(i,k)*x(k)
  enddo
enddo
return
end

subroutine vec_mag(x,n,x_mag)
real,intent(in) :: x(n)
integer,intent(in) :: n
real,intent(out) :: x_mag
integer :: i
x_mag = 0.0
do i = 1,n
  x_mag = x_mag + x(i)*x(i)
enddo
x_mag = sqrt(x_mag)
return
end

program eigen_large
parameter (n=3,nit_max=28)
real :: a(n,n),x(n),y(n),r(n),s(n)
data a/2.0,-2,-1.0,0.0,-10.0,-1.0,-1.0,0.0,4.0/
data x/-1.0,1.0,1.0/
!
call power_aitken(a,n,x,y,r,nit_max,s)
end program eigen_large



  
  
